<?php
/**
 *
 * Auther: heyu
 * DateTime: 2019-12-24 13:56
 *
 */

namespace kwy\aliPayOpenApi\lib;


use kwy\util\http\CurlHttp;

class ApiRequest
{
    /** @var CurlHttp $http */
    private $http;

    /**
     * @return CurlHttp
     */
    public function getHttp(): CurlHttp
    {
        if (empty($this->http)){
            $this->http = new CurlHttp();
        }
        return $this->http;
    }

    /**
     * @param CurlHttp $http
     */
    public function setHttp(CurlHttp $http)
    {
        $this->http = $http;
    }
    /** @var Config $config */
    private $config;

    /**
     * @return Config
     */
    public function getConfig(): Config
    {
        return $this->config;
    }

    /**
     * @param Config $config
     */
    public function setConfig(Config $config)
    {
        $this->config = $config;
    }
    protected function call($method,$biz_content=[],$data=[]){
        $url=Config::GATE_WAY.'?charset=utf-8';

        $http=$this->getHttp();
        $http->setUrl($url);
        $biz_content['service_id'] = $this->config->app_id;
        $params=[
            'app_id'=>$this->config->app_id,
            'method'=>$method,
            'charset'=>'utf-8',
            'sign_type'=>'RSA2',
            'timestamp'=>date('Y-m-d H:i:s'),
            'version'=>'1.0',
            'biz_content'=>json_encode($biz_content),
        ];
        foreach($data as $k=>$v){
            $params[$k] = $v;
        }
        if (!empty($this->config->app_auth_token)){
            $params['app_auth_token']=$this->config->app_auth_token;
        }
        $params['sign']=$this->getSign($params);
        $http->setResponseType('array');
        return $http->post($params);
    }

    /**
     * 设置代理服务器
     * @author roc
     * @datetime:2020/3/14 17:24
     * @param $proxy
     * @return CurlHttp
     */
    public function setHttpProxy($proxy){
        $http=$this->getHttp();
        $http->setProxy($proxy);
        return $this->http;
    }

    /**
     * 获取签名
     * @author roc
     * @datetime:2020/3/14 17:22
     * @param $params
     * @return string
     * @throws \Exception
     */
    private function getSign($params){
        $str='';
        ksort($params);
        foreach ($params as $k=>$v){
            $str.=$k.'='.$v.'&';
        }
        $str = rtrim($str,'&');
        if (!empty($this->config->private_key_path)){
            $priKey = file_get_contents($this->config->private_key_path);
        }else{
            $priKey = empty($this->config->private_key_content)?false:$this->config->private_key_content;
        }

        if ($priKey===false){
            throw new \Exception('证书文件路径错误或者传入的证书格式错误');
        }
        $proKeyRes = openssl_get_privatekey($priKey);
        $sign='';
        openssl_sign($str, $sign, $proKeyRes, OPENSSL_ALGO_SHA256);
        openssl_free_key($proKeyRes);
        return base64_encode($sign);
    }

}